/*
 * Bluemote - use your Bluetooth enabled T610 or compatible Sony Ericsson phone as
 * a remote for your PC.
 * Copyright (C) 2004
 * Saravana K Kannan <saravkrish at uky dot edu>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "parse.h"

/* Global variables */
VAR var[VARCNT];
int varcnt=0;

CONF defconf = { FALSE, FALSE, 60, FALSE, "", "/dev/rfcomm0",
		"aumix-minimal -w",
		"aumix-minimal -q | grep pcm | awk '{print $3}'",
		-1, NULL, -1, NULL, -1, NULL };

/* Set the value of a variable */
void setvar(char *vname, char *val)
{
 int i=0;

 while(strcmp(var[i].name,vname) && i<varcnt) i++;
 if(i==varcnt && i<VARCNT)
 {
  strcpy(var[i].name,vname);
  varcnt++;
 }
 if(i<varcnt)
  strcpy(var[i].val, val);
}

/* Get the value of a variable */
char *getvar(char *vname)
{
 int i=0;
 
 while(strncmp(var[i].name,vname, VARLEN) && i<varcnt) i++;
 if(i==varcnt)
  return vname;
 else
  return var[i].val;
}

void substvar(char *strlist, int maxargs, int maxarglen)
{
 int i;
 for(i=0; i < maxargs; i++)
  strncpy(strlist+i*maxarglen, getvar(strlist+i*maxarglen), maxarglen);
}

/* Parses command line options and set flags. */
void parseopts(int argc, char *argv[], CONF *conf)
{
 int i;
 for(i=1; i<argc; i++)
 {
  if(strcmp(argv[i],"--log")==0) conf->log=TRUE;
  else if(strcmp(argv[i],"--daemon")==0) conf->daemon=TRUE;
  else if(strcmp(argv[i],"--send")==0) conf->msg=TRUE;
  else strcpy(conf->device, argv[i]);
 }
}

/* Splits 'input' string into individual argments and stores them in the 'args'
 * array of strings. */
int split(char *input, char *args, int maxargs, int maxarglen)
{
 int mode=0;
 int i=0, argcnt;
 char ch, tmp[MAXARGLEN];

 argcnt=0;
 while(*input && *input != '\n' && argcnt < maxargs)
 {
  ch=*input;

  switch(mode)
  {
   /* Mode for walking past delimeters ' ' and '\t' */
   case 0:
   	if(ch==' ' || ch=='\t')
   	 break;
	else
	{
	 if(ch=='\"')
	 {
	  mode=2;
	  input++;
	  continue;
	 }
	 else mode=1;
	}
	break;
   
   /* Mode for reading in single word arguments */
   case 1:
   	if(ch==' ' || ch=='\t')
	 mode=3;
   	break;
   
   /* Mode for reading in multi-word arguments that
    * are enclosed within "" */
   case 2:
	if(ch=='\"')
	 mode=3;
	/* A \ escapes any character following it. Even a " */
	if(ch=='\\')
	{
	 input++;
	 ch=*input;
	}
   	break;
  }

  /* Mode where the whole argument is in tmp and can be copied
   * into args[argcnt] */
  if(mode==3)
  {
   tmp[i]=0;
   strncpy(args+(argcnt*maxarglen),tmp,maxarglen);
   argcnt++;
   mode=0;
   i=0;
  }
  if(mode && i < (MAXARGLEN-1)) tmp[i++]=ch;
  input++;
 }

 /* Copy any last left out argument */
 if(mode && argcnt < maxargs)
 {
  tmp[i]=0;
  strncpy(args+(argcnt*maxarglen),tmp,maxarglen);
  argcnt++;
 }

 return argcnt;
}

/* Returns true if the line can be ignored in the config file. */
int ignoreline(char *line)
{
 if((*line)==';') return 1;
 while(isspace(*line) && *line) line++;
 return !(*line);
}

/* Load a section containing name and value pairs. */
VAR* loadvarsection(FILE *fp, int *count)
{
 VAR *tvar = NULL;
 char tmptext[MAXLEN], *ret, option[10][MAXARGLEN];
 int varcnt, optioncnt, i, j;
 long pos;

 /* Count the number of variables. */
 varcnt=0;
 pos=ftell(fp);
 do
 {
  ret=fgets(tmptext, MAXLEN, fp);
  if(ret==NULL)
   return NULL;
  if(ignoreline(tmptext)) continue;
  optioncnt = split(tmptext, option[0], 2, MAXARGLEN);
  if(optioncnt >= 2) varcnt++;
 }while(strncmp("[End]\n",tmptext,MAXLEN)!=0);
 fseek(fp, pos, SEEK_SET);

 tvar = (VAR *) calloc(varcnt, sizeof(VAR));
 i=0;
 while(i < varcnt)
 {
  ret=fgets(tmptext, MAXLEN, fp);
  if(ret==NULL)
  {
   free(tvar);
   return NULL;
  }
  if(ignoreline(tmptext)) continue;
  optioncnt = split(tmptext, option[0], 10, MAXARGLEN);
  if(optioncnt >= 2)
  {
   strncpy(tvar[i].name, option[0], VARLEN);
   strncpy(tvar[i].val, option[1], MAXLEN);
   for(j=2; j < optioncnt; j++)
   {
    strncat(tvar[i].val, " ", MAXLEN - strlen(tvar[i].val));
    strncat(tvar[i].val, option[j], MAXLEN - strlen(tvar[i].val));
   }
   i++;
  }
 }

 if(i < varcnt)
 {
  free(tvar);
  return NULL;
 }
 else
 {
  *count = varcnt;
  return tvar;
 }
}

/* Load a section from the file and return an arroy of MENU. Returns NULL on
 * error. */
MENU* loadmenusection(FILE *fp, int *count)
{
 MENU *tmenu = NULL;
 char tmptext[MAXLEN], *ret;
 int i, j, mode=0, type=0, menucnt;
 long pos;

 /* Count the number of menu items. */
 menucnt=0;
 pos=ftell(fp);
 do
 {
  ret=fgets(tmptext, MAXLEN, fp);
  if(ret==NULL)
   return NULL;
  if(ignoreline(tmptext)) continue;
  if(tmptext[0]!='\t') menucnt++;
 }while(strncmp("[End]\n",tmptext,MAXLEN)!=0);

 /* Adjust for counting [End] as a menu item. */
 menucnt--;
 fseek(fp, pos, SEEK_SET);
 
 /* Read in the menu section. */
 tmenu = (MENU *) calloc(menucnt, sizeof(MENU));
 i=0;
 while(i < menucnt)
 {
  ret=fgets(tmptext, MAXLEN, fp);
  if(ret==NULL)
  {
   free(tmenu);
   return NULL;
  }
  if(ignoreline(tmptext)) continue;
  tmptext[strlen(tmptext)-1]=0;
  if(tmptext[0]=='\t') type = TYPE_CMD;
  else if(strncmp("[End]\n",tmptext,MAXLEN)==0) type = TYPE_END;
  else type = TYPE_TXT;

  if(mode==1)
  {
   /* End of commands for the current menu item. */
   if(j==MAXMCMD || type != TYPE_CMD)
   {
    tmenu[i].ncmd=j;
    mode=0;
    i++;
   }
   else // Note down the commands for a particular menu item.
    strncpy(tmenu[i].cmd[j++], tmptext+1, MCMDLEN);
  }
  
  /* Note down the text for a new menu item. */
  if(mode==0)
  {
   if(type != TYPE_TXT) break;
   strncpy(tmenu[i].text, tmptext, MTEXTLEN);
   mode=1;
   j=0;
  }
 }
 
 if(i < menucnt)
 {
  free(tmenu);
  return NULL;
 }
 else
 {
  *count = menucnt;
  return tmenu;
 }
}

/* Parses the config file and returns a CONF data structure. Returns a NULL on
 * error. */
CONF* loadcfg(char *mfile)
{
 FILE *fp;
 CONF *conf = NULL;
 char tmptext[MAXLEN], *ret;
 char option[2][MAXARGLEN];
 int optioncnt;

 fp=fopen(mfile,"r");
 if(fp==NULL) return NULL;
 
 conf = (CONF *) malloc(sizeof(CONF));
 *conf = defconf;

 /* Go through the conf file. */
 while(TRUE)
 {
  ret=fgets(tmptext, MAXLEN, fp);
  if(ret==NULL) break;
  optioncnt = split(tmptext, option[0], 2, MAXARGLEN);
  
  /* Parse general options. */
  if(optioncnt == 2)
  {
   if(strncmp("Log",option[0],MAXARGLEN)==0 
      && strncmp("Yes",option[1],MAXARGLEN)==0)
	conf->log=1;
   
   if(strncmp("Daemon",option[0],MAXARGLEN)==0 
      && strncmp("Yes",option[1],MAXARGLEN)==0)
	conf->daemon=1;
   
   if(strncmp("CharSet",option[0],MAXARGLEN)==0)
   	strncpy(conf->charset,option[1],MAXARGLEN);

   if(strncmp("RetrySecs",option[0],MAXARGLEN)==0)
   	conf->retrysecs=atoi(option[1]);

   if(strncmp("SetVol",option[0],MAXARGLEN)==0)
   	strncpy(conf->setvol,option[1],MAXARGLEN);

   if(strncmp("GetVol",option[0],MAXARGLEN)==0)
   	strncpy(conf->getvol,option[1],MAXARGLEN);

   if(strncmp("Device",option[0],MAXARGLEN)==0)
   	strncpy(conf->device,option[1],MAXARGLEN);
  }
  
  /* Parse sections. */
  if(optioncnt==1)
  {
   if(strncmp("[Menu]",option[0],MAXARGLEN)==0 && conf->menu== NULL)
    conf->menu = loadmenusection(fp,&(conf->menucnt));
   if(strncmp("[Event]",option[0],MAXARGLEN)==0 && conf->event== NULL)
    conf->event = loadmenusection(fp,&(conf->eventcnt));
   if(strncmp("[KeyMap]",option[0],MAXARGLEN)==0 && conf->keymap== NULL)
    conf->keymap = loadvarsection(fp,&(conf->keymapcnt));
  }
 }

 fclose(fp);

 if(conf->menu == NULL)
 {
  freeconf(conf);
  return NULL;
 }

 return conf;
}

void freeconf(CONF *conf)
{
 if(conf==NULL) return;
 
 /* Passing NULL to free() is okay. */
 free(conf->menu);
 free(conf->event);
 free(conf->keymap);
 free(conf);
}

/* Form a command string made up of concatenated arguments. The string from 
 * pmenu[].cmd[] can't be used since the $ variables will be in unsubstituted
 * form. */
void parsecmd(char *cmd, char *out)
{
 int i, argcnt;
 char args[MAXARGS][MAXARGLEN];

 out[0]=0;
 argcnt = split(cmd, args[0], MAXARGS, MAXARGLEN);
 for(i=0; i<argcnt; i++)
 {
  strcat(out, getvar(args[i]));
  if(i!=(argcnt-1)) strcat(out, " ");
 }
}

void printcfg(CONF *conf)
{
 int i;
 char args[MAXARGS][MAXARGLEN];
 int argcnt = 0;

 printf("Device: %s\nRetry: %d\nLog: %s\n\n", conf->device, conf->retrysecs, (conf->log?"Yes":"No"));

 printf("Menu Entries:\n");
 for(i=0; i<conf->menucnt; i++)
 {
  printf("Text: %s\n", conf->menu[i].text);
  int j;
  for(j=0; j<conf->menu[i].ncmd; j++)
   printf("\t%s\n",conf->menu[i].cmd[j]);
 }

 printf("\nEvent Entries:\n");
 for(i=0; i<conf->eventcnt; i++)
 {
  printf("Text: %s\n", conf->event[i].text);
  int j;
  for(j=0; j<conf->event[i].ncmd; j++)
   printf("\t%s\n",conf->event[i].cmd[j]);
 }

 printf("\nKey Mapping:\n");
 for(i=0; i<conf->keymapcnt; i++)
  printf("%s = %s\n", conf->keymap[i].name, conf->keymap[i].val);

 printf("\nsplit() test:\n");
 printf("Input: %s\n",TEST_STR_SPLIT);
 argcnt = split(TEST_STR_SPLIT, args[0], MAXARGS, MAXARGLEN);
 for(i=0; i<argcnt; i++)
  printf("%d: --%s--\n",i,args[i]);
}
