/*
 * Bluemote - use your Bluetooth enabled T610 or compatible Sony Ericsson phone as
 * a remote for your PC.
 * Copyright (C) 2004
 * Saravana K Kannan <saravkrish at uky dot edu>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "bluemote.h"
#include "serial.h"
#include "ui.h"

int nstate=1;

void ui_nextstate(int nextstate)
{
 nstate = nextstate;
}

int status(char *msg)
{
 char cmd[MAXATCMDLEN];

 sprintf(cmd, "AT*EAST=\"%s\"\r", msg);
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 return 1;
}

int nodialog()
{
 char cmd[MAXATCMDLEN] = "AT*EAID=0\r";

 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 return 1;
}

int msgbox(char *msg, unsigned int timeout)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 sprintf(cmd, "AT*EAID=1,%d,\"%s\",%d\r", nstate, msg, timeout);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 1,1", 10)==0) return 1;
 
 return -1;
}

int get_yesno(char *ques)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 sprintf(cmd, "AT*EAID=2,%d,\"%s\"\r", nstate, ques);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 2,0", 10)==0) return 1;
 if(strncmp(buf, "*EAII: 2,1", 10)==0) return 2;
 
 return -1;
}

int get_onoff(char *ques, unsigned int def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(def) def=1;

 sprintf(cmd, "AT*EAID=3,%d,\"%s\",%d\r", nstate, ques, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 3,0", 10)==0) return 1;
 if(strncmp(buf, "*EAII: 3,1", 10)==0) return 2;
 
 return -1;
}

int get_percent(char *msg, unsigned int parts, unsigned int def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 sprintf(cmd, "AT*EAID=4,%d,\"%s\",%d,%d\r", nstate, msg, parts, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR || strncmp(buf,"*EAII: 15,",10)==0);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 4,", 9)==0)
  return (atoi(buf+9)+1);
 
 return -1;
}

int choicecnt;
char choicetxt[MAXATCMDLEN];

void init_choice()
{
 choicecnt=0;
 choicetxt[0]=0;
}

void addchoice(char *choice)
{
 strcat(choicetxt,",\"");
 strcat(choicetxt,choice);
 strcat(choicetxt,"\"");
 choicecnt++;
}

int get_choice(char *msg, unsigned int def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(def==0 || def > choicecnt) def=1;
 sprintf(cmd, "AT*EAID=5,%d,\"%s\",%d,%d%s\r", nstate, msg, def, choicecnt, choicetxt);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 5,", 9)==0)
  return atoi(buf+9);

 return -1;
}

int get_real(char *out, char *title, char *msg, char *max, char *def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";
 if(msg==NULL) msg="";
 if(max==NULL) return -1;
 if(def==NULL) def="";

 sprintf(cmd, "AT*EAID=6,%d,\"%s\",\"%s\",\"%s\",\"%s\"\r", nstate, title, msg, max, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 6,\"", 10)==0)
 {
  out[0]=0;
  strncat(out, buf+10, strchr(buf+10,'\"')-(buf+10));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_integer(char *out, char *title, char *msg, int min, int max, int def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";
 if(msg==NULL) msg="";
 if(def<min || def>max) def=min;

 sprintf(cmd, "AT*EAID=7,%d,\"%s\",\"%s\",%d,%d,%d\r", nstate, title, msg, min, max, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 7,", 9)==0)
 {
  sprintf(out, "%d", atoi(buf+9));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_phone(char *out, char *title, char *msg, char *def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";
 if(msg==NULL) msg="";
 if(def==NULL) def="";

 sprintf(cmd, "AT*EAID=8,%d,\"%s\",\"%s\",\"%s\"\r", nstate, title, msg, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 8,\"", 10)==0)
 {
  out[0]=0;
  strncat(out, buf+10, strchr(buf+10,'\"')-(buf+10));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_date(char *out, char *msg)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(msg==NULL) msg="";

 sprintf(cmd, "AT*EAID=9,%d,\"%s\"\r", nstate, msg);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 9,\"", 10)==0)
 {
  out[0]=0;
  strncat(out, buf+10, strchr(buf+10,'\"')-(buf+10));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_time(char *out, char *msg, char *def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(msg==NULL) msg="";
 if(def==NULL) def="0:00";

 sprintf(cmd, "AT*EAID=10,%d,\"%s\",\"%s\"\r", nstate, msg, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 10,\"", 11)==0)
 {
  out[0]=0;
  strncat(out, buf+11, strchr(buf+11,'\"')-(buf+11));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_text(char *out, char *title, char *msg, unsigned int maxlen, char *def)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";
 if(msg==NULL) msg="";
 if(def==NULL) def="";

 sprintf(cmd, "AT*EAID=11,%d,\"%s\",\"%s\",%d,\"%s\"\r", nstate, title, msg, maxlen, def);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 11,\"", 11)==0)
 {
  out[0]=0;
  strncat(out, buf+11, strrchr(buf+11,'\"')-(buf+11));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int get_secret(char *out, char *title, char *msg, unsigned int maxlen)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";
 if(msg==NULL) msg="";

 sprintf(cmd, "AT*EAID=12,%d,\"%s\",\"%s\",%d\r", nstate, title, msg, maxlen);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 12,\"", 11)==0)
 {
  out[0]=0;
  strncat(out, buf+11, strchr(buf+11,'\"')-(buf+11));
  return 1;
 }
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 
 return -1;
}

int progressbar(char *msg)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 sprintf(cmd, "AT*EAID=13,%d,\"%s\"\r", nstate, msg);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 1;
 
 return -1;
}

int info(char *title, char *info)
{
 int ret;
 char cmd[MAXATCMDLEN];
 char buf[MAXATCMDLEN]="";

 if(title==NULL) title="";

 sprintf(cmd, "AT*EAID=14,%d,\"%s\",\"%s\"\r", nstate, title, info);
 
 writeport(cmd, strlen(cmd));
 if(waitfor("OK", 1, TIMEOUT) != 1) return -1;

 while((ret = readport(buf, MAXATCMDLEN, TIMEOUT)) == -EINTR);
 if(ret <= 0) return -1;
 if(strncmp(buf, "*EAII: 0", 8)==0) return 0;
 if(strncmp(buf, "*EAII: 14,1", 11)==0) return 1;
 
 return -1;
}

int ui_test()
{
 char output[MAXATCMDLEN]="Test message";

	msgbox(output,0);
	printf("Yes/No: %d\n",get_yesno("Working?"));
	printf("On/Off: %d\n",get_onoff("Sound",0));
	printf("Percent: %d\n",get_percent("Volume",10,7));
	init_choice();
	addchoice("S");
	addchoice("M");
	addchoice("L");
	addchoice("XL");
	printf("Choice: %d\n",get_choice("Size", 3));
	printf("Real: %d\n", get_real(output, "Title","Msg","1.9","1.5"));
	printf("Real: %s\n",output);
	printf("Integer: %d\n", get_integer(output, "Title","Msg",-15,19,0));
	printf("Integer: ==%s==\n",output);
	printf("Phone: %d\n", get_phone(output, "Phone", "No.", "4897331"));
	printf("Phone: %s\n",output);
	printf("Date: %d\n", get_date(output, "BDay"));
	printf("Date: %s\n",output);
	printf("Time: %d\n", get_time(output, "TiMe", "21:00"));
	printf("Time: %s\n",output);
	printf("Text: %d\n", get_text(output, "Title", "Letter", 100, "Hi,"));
	printf("Text: %s\n",output);
	printf("PIN: %d\n", get_secret(output, "Blueremote", "PIN", 10));
	printf("PIN: %s\n",output);
	printf("Info: %d\n", info("Title", "This is a lot of information to fit in a small area. Lessee if it can handle this. Hmm.. It can. Lets try something even bigger! Hope this should be enuf!"));

 return 0;
}

